/* Copyright 2020 The MathWorks, Inc. */

#overlay {
    position: fixed; /* Sit on top of the page content */
    width: 100%; /* Full width (cover the whole page) */
    height: 100%; /* Full height (cover the whole page) */
    top: 0;
    left: 0;
    right: 0;
    bottom: 0;
    background-color: rgba(0,0,0,0.5); /* Black background with opacity */
    z-index: 2; /* Specify a stack order in case you're using a different order for other elements */
    transition: opacity 0.5s ease-out, visibility 0.5s ease-out;
    visibility: visible;
}

label {
    font-weight: normal;
    font-size: 18px;
    margin-bottom: 10px;
}

.has-feedback label~.form-control-feedback {
    top: 35px;
}
