// Copyright 2020 The MathWorks, Inc.

export const SET_TRIGGER_POSITION = 'SET_TRIGGER_POSITION';
export const SET_TUTORIAL_HIDDEN = 'SET_TUTORIAL_HIDDEN';
export const SET_OVERLAY_VISIBILITY = 'SET_OVERLAY_VISIBILITY';
export const REQUEST_SERVER_STATUS = 'REQUEST_SERVER_STATUS';
export const RECEIVE_SERVER_STATUS = 'RECEIVE_SERVER_STATUS';
export const REQUEST_SET_LICENSING = 'REQUEST_SET_LICENSING';
export const REQUEST_TERMINATE_INTEGRATION = 'REQUEST_TERMINATE_INTEGRATION';
export const REQUEST_STOP_MATLAB = 'REQUEST_STOP_MATLAB';
export const REQUEST_START_MATLAB = 'REQUEST_START_MATLAB';
export const RECEIVE_SET_LICENSING = 'RECEIVE_SET_LICENSING';
export const RECEIVE_TERMINATE_INTEGRATION = 'RECEIVE_TERMINATE_INTEGRATION';
export const RECEIVE_STOP_MATLAB = 'RECEIVE_STOP_MATLAB';
export const RECEIVE_START_MATLAB = 'RECEIVE_START_MATLAB';
export const RECEIVE_ERROR = 'RECEIVE_ERROR';
export const REQUEST_ENV_CONFIG = 'REQUEST_ENV_CONFIG';
export const RECEIVE_ENV_CONFIG = 'RECEIVE_ENV_CONFIG';
